/*
based on material Copyright (C) 2013 Holtek Semiconductor Inc. All rights reserved
*/

/* Includes ------------------------------------------------------------------------------------------------*/
#include <stddef.h>		// for offsetof
#include "ht32_usbd_cdcclass.h"

/* Private constants ---------------------------------------------------------------------------------------*/
// class-specific requests
#define CDCCLASS_REQ_SET_LINE_CODING        (0x20)
#define CDCCLASS_REQ_GET_LINE_CODING        (0x21)
#define CDCCLASS_REQ_SET_CONTROL_LINE_STATE (0x22)
// subclass-specific requests
// subclass-specific notifications
#define CDCCLASS_NOT_PSTN_SERIAL_STATE (0x20)

/* Private typedef -----------------------------------------------------------------------------------------*/
/* Global variables ----------------------------------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------------------------------------*/
// this requires --diag_suppress=170 in Keil
static volatile uint8_t rxready = 0, txbusy = 0;


static USBDCDCClass_LINE_CODING USBDCDCClassLineCoding;
static vu32 mysignals = 0, myoldsignals = 0; ///< quick, dirty, ugly and shameless control signal emulation

/* Private macro -------------------------------------------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------------------------------------*/
static void USBDClass_MainRoutine(u32 uPara);

static void USBDClass_Request(USBDCore_Device_TypeDef *pDev);
static void USBDClass_SetLineCoding(USBDCore_Device_TypeDef *pDev);
static void USBDClass_GetLineCoding(USBDCore_Device_TypeDef *pDev);
static void USBDClass_SetControlLineState(USBDCore_Device_TypeDef *pDev);
static void USBDClass_Endpoint1(USBD_EPTn_Enum EPTn);
static void USBDClass_Endpoint2(USBD_EPTn_Enum EPTn);
static void USBDClass_Endpoint3(USBD_EPTn_Enum EPTn);
//static void USBDClass_Reset(u32 uPara);
//static void USBDClass_StartOfFrame(u32 uPara);
//static void USBDClass_Standard_GetDescriptor(USBDCore_Device_TypeDef *pDev);
//static void USBDClass_Standard_SetInterface(USBDCore_Device_TypeDef *pDev);
//static void USBDClass_Standard_GetInterface(USBDCore_Device_TypeDef *pDev);

/* Global Function -----------------------------------------------------------------------------------------*/
/*********************************************************************************************************//**
  * @brief  USB Class initialization.
  * @param  pClass: pointer of USBDCore_Class_TypeDef
  * @retval None
  ***********************************************************************************************************/
void USBDCDCClass_Init(USBDCore_Class_TypeDef *pClass)
{
	pClass->CallBack_MainRoutine.func = USBDClass_MainRoutine;
	pClass->CallBack_MainRoutine.uPara = (u32)0;
	
	//pClass->CallBack_Reset.func = USBDClass_Reset;
	//pClass->CallBack_Reset.uPara = (u32)NULL;
	
	//pClass->CallBack_StartOfFrame.func = USBDClass_StartOfFrame;
	//pClass->CallBack_StartOfFrame.uPara = (u32)NULL;
	
	//pClass->CallBack_ClassGetDescriptor = USBDClass_Standard_GetDescriptor
	//pClass->CallBack_ClassSetInterface = USBDClass_Standard_SetInterface;
	//pClass->CallBack_ClassGetInterface = USBDClass_Standard_GetInterface;
	
	
	pClass->CallBack_ClassRequest = USBDClass_Request;
	pClass->CallBack_EPTn[1] = USBDClass_Endpoint1;
	pClass->CallBack_EPTn[2] = USBDClass_Endpoint2;
	pClass->CallBack_EPTn[3] = USBDClass_Endpoint3;
}

CDCclass_send(u32 *buffer, int len)
{
	if (txbusy)
		return(-1);
	txbusy = 1;
	USBDCore_EPTWriteINData(USBD_EPT1, buffer, len);
	return(1);
}

CDCclass_receive(u32 *buffer)
{
	if (rxready){
		rxready = 0;
		return(USBDCore_EPTReadOUTData(USBD_EPT3, buffer, _EP3LEN));
	}
	return(-1);
}

/* Private functions ---------------------------------------------------------------------------------------*/
/*********************************************************************************************************//**
  * @brief  USB Class main routine.
    Endpoint 2 IN Process for control signals
  * @param  uPara: Parameter for Class main routine
  * @retval None
  ***********************************************************************************************************/
typedef __PACKED_H struct
{
	u8 bmRequestType;
	u8 bNotification;
	u16 wValue;
	u16 wIndex;
	u16 wLength;
	u16 Data[1];					///< define suitable buffer length here
} __PACKED_F CDCclass_Notification_TypeDef;

#define CDCCLASS_NOTHEADER_LEN (offsetof(CDCclass_Notification_TypeDef,Data))

static void USBDClass_MainRoutine(u32 uPara)
{
static CDCclass_Notification_TypeDef n;

	if (mysignals != myoldsignals) {	///< detect signal change
		myoldsignals = mysignals;
		n.bmRequestType = REQ_DIR_01_D2H | REQ_TYPE_01_CLS | REQ_REC_01_INF; // 0xA1
		n.bNotification = CDCCLASS_NOT_PSTN_SERIAL_STATE;
		n.wValue = 0;
		n.wIndex = 0;
		n.wLength = 2;
		n.Data[0] = (u16) mysignals;	///< insert real signals here
		USBDCore_EPTWriteINData(USBD_EPT2, (u32 *)&n, CDCCLASS_NOTHEADER_LEN + n.wLength);	 ///< and send
	}
}

/*********************************************************************************************************//**
  * @brief  USB Device Class Request
  * @param  pDev: pointer of USB Device
  * @retval None
  ***********************************************************************************************************/
static void USBDClass_Request(USBDCore_Device_TypeDef *pDev)
{
	u8 USBCmd = *((u8 *)(&(pDev->Request.bRequest)));
	u16 len = *((u16 *)(&(pDev->Request.wLength)));

	switch (USBCmd)	{
		case CDCCLASS_REQ_SET_CONTROL_LINE_STATE:
			if(len == 0)
				USBDClass_SetControlLineState(pDev);
			break;
		case CDCCLASS_REQ_SET_LINE_CODING:
			USBDClass_SetLineCoding(pDev);
			break;
		case CDCCLASS_REQ_GET_LINE_CODING:
			USBDClass_GetLineCoding(pDev);
			break;
		// others not supported
		default:
			break;
	}
}

/*********************************************************************************************************//**
  * @brief  USB Device Class Request - Set Line Coding
  * @param  pDev: pointer of USB Device
  * @retval None
  ***********************************************************************************************************/
static void USBDClass_SetLineCoding(USBDCore_Device_TypeDef *pDev)
{
	pDev->Transfer.pData = (uc8*)&USBDCDCClassLineCoding;
	pDev->Transfer.sByteLength = (sizeof(USBDCDCClassLineCoding) > pDev->Request.wLength) ? (pDev->Request.wLength) : (sizeof(USBDCDCClassLineCoding));
	pDev->Transfer.Action = USB_ACTION_DATAOUT;
}

/*********************************************************************************************************//**
  * @brief  USB Device Class Request - Get Line Coding
  * @param  pDev: pointer of USB Device
  * @retval None
  ***********************************************************************************************************/
static void USBDClass_GetLineCoding(USBDCore_Device_TypeDef *pDev)
{
	pDev->Transfer.pData = (uc8*)&USBDCDCClassLineCoding;
	pDev->Transfer.sByteLength = (sizeof(USBDCDCClassLineCoding) > pDev->Request.wLength) ? (pDev->Request.wLength) : (sizeof(USBDCDCClassLineCoding));
	pDev->Transfer.Action = USB_ACTION_DATAIN;
}

/*********************************************************************************************************//**
  * @brief  USB Device Class Request - Set Control Line State
  * @param  pDev: pointer of USB Device
  * @retval None
  ***********************************************************************************************************/
static void USBDClass_SetControlLineState(USBDCore_Device_TypeDef *pDev)
{
	mysignals = pDev->Request.wValueL;	///< get signals and store to loop them back
	pDev->Transfer.pData = 0;
	pDev->Transfer.sByteLength =0;
	pDev->Transfer.Action = USB_ACTION_DATAOUT;
}

/*********************************************************************************************************//**
  * @brief  USB Class Endpoint handler
  * @param  EPTn: USB Endpoint number
  *         @arg USBD_EPT0 ~ USBD_EPT7
  * @retval None
  ***********************************************************************************************************/
static void USBDClass_Endpoint1(USBD_EPTn_Enum EPTn)
{
	txbusy = 0;
}

/*********************************************************************************************************//**
  * @brief  USB Class Endpoint handler
  * @param  EPTn: USB Endpoint number
  *         @arg USBD_EPT0 ~ USBD_EPT7
  * @retval None
  ***********************************************************************************************************/
static void USBDClass_Endpoint2(USBD_EPTn_Enum EPTn)
{
}

/*********************************************************************************************************//**
  * @brief  USB Class Endpoint handler
  * @param  EPTn: USB Endpoint number
  *         @arg USBD_EPT0 ~ USBD_EPT7
  * @retval None
  ***********************************************************************************************************/
static void USBDClass_Endpoint3(USBD_EPTn_Enum EPTn)
{
	rxready = 1;
}

