#include <ht32f175x_275x.h>                    // 

#include <string.h>

#include "ht32_usbd_core.h"
#include "ht32_usbd_cdcclass.h"
#include "ht32_usbd_cdcdescriptor.h"

/* Global variables ----------------------------------------------------------------------------------------*/
__ALIGN4 USBDCore_TypeDef gUSBCoreVCP;
USBD_Driver_TypeDef gUSBDriver;

void Suspend(u32 uPara);


/* Development board needs to set a GPIO to enable USB pull-up
in order for the device to say "here I am" */
void HT32F175xDVB_USBConnect(void)
{
	HT_CKCU->APBCCR0 |= (1<<14)+(1<<16);	// 14=AFIOEN, 16=PAEN, enable APB clocks for AFIO and GPIOA
	HT_AFIO->GPACFGR &= ~(0x03<<8);		// PA4 -> AF0 (PA4)
	HT_GPIOA->DIRCR |= 0x0010;		// PA4 defined as Output
	HT_GPIOA->DOUTR |= (1<<4);		// PA4=1
}

void init_USBstuff(void);

/* MAIN PROGRAM */

main()
{
__ALIGN4 static u8 inbuff[64];	///< size must accomodate endpoint buffer
__ALIGN4 static u8 outbuff[64];
int ret, state=0;

	HT_CKCU->APBCCR1 |= (1<<14);		// 14=USBEN, enable APB clock for USB registers/RAM
	HT_CKCU->GCFGR &= ~(0x03<<22);		// clear USB clock prescaler bits
	// see PLL and AHB clock definition in system_.c, we need 48MHz for the USB peripheral
	HT_CKCU->GCFGR |= (0x02<<22);		// set USB clock to PLL/3 (144/3 = 48MHz)

	init_USBstuff();			// init USB stuff
	
	while(1){
		USBDCore_MainRoutine(&gUSBCoreVCP);	// handle USB stuff and do callback calls for each class
		switch(state){
		case 0:	//idle
			if((ret=CDCclass_receive((u32 *)inbuff))!=-1){
				// got msg!;
				memcpy(outbuff,inbuff,ret);	// loopback
				state = 1;
			}
			break;
		case 1:	// send msg
			if(CDCclass_send((u32 *)outbuff, ret) != -1) ///< msg length must not exceed endpoint buffer length
				state = 0;
			break;
		}
	}
}

void init_USBstuff()
{
	gUSBCoreVCP.pDriver = (u32 *)&gUSBDriver;
	// Configure the function to be called on USB suspend state
	gUSBCoreVCP.Power.CallBack_Suspend.func  = Suspend;
	//gUSBCore.Power.CallBack_Suspend.uPara = NULL;

	// Init pointer to descriptor defined in ht32_usbd_cdcdescriptor.c
	USBDCDCDesc_Init(&gUSBCoreVCP.Device.Desc);
		// Init pointer to class defined in ht32_usbd_cdcclass.c
	USBDCDCClass_Init(&gUSBCoreVCP.Class);
/* USB data in/out for this class will be handled by functions inside
ht32_usbd_cdcclass.c, called back by USBDCore_MainRoutine() */
	// Init Hardware
	USBDCore_Init(&gUSBCoreVCP);
	// Enable interrupts  
	NVIC_EnableIRQ(USB_IRQn);
	// enable pull-up in USB D+ (say "here I am")
	HT32F175xDVB_USBConnect();
	// wait for enumeration	
}

/* USB suspend event will generate a call to this function via 
a USB Power handler function called by USBDCore_MainRoutine() */
void Suspend(u32 uPara)
{
}

/* USB interrupts are derived to USBDCore_IRQHandler(); which in turn
will check and call each endpoint interrupt, implemented inside
ht32_usbd_cdcclass.c */
void USB_IRQHandler(void)
{
__ALIGN4 extern USBDCore_TypeDef gUSBCoreVCP;

	USBDCore_IRQHandler(&gUSBCoreVCP);
}
